@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2020-2024 Intel Corporation
+Copyright (C) 2020-2025 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -35,6 +35,8 @@ SPDX-License-Identifier: MIT
 // The others are lower level internal utilities, used only by higher level macros.
 //
 // Explanation of code in original resource : https://www.fluentcpp.com/2019/08/30/how-to-disable-a-warning-in-cpp/
+
+// clang-format off
 #if defined(__clang__)
     #define DO_PRAGMA_DIAG(X) _Pragma(#X)                                                     // internal utility
     #define DISABLE_WARNING_PUSH         DO_PRAGMA_DIAG(GCC diagnostic push)
@@ -78,10 +80,11 @@ SPDX-License-Identifier: MIT
     #define DISABLE_WARNING_PADDING_AT_END_OF_STRUCT
     #define DISABLE_WARNING_ANON_TYPES_IN_ANON_UNION
 #endif
+// clang-format on
 
-
-DISABLE_WARNING_PUSH       // save the current pragma state, save the current compiler settings
-// select the warnings that we want to disable, for this file only
+// save the current pragma state, save the current compiler settings select the
+// warnings that we want to disable, for this file only
+DISABLE_WARNING_PUSH
 DISABLE_WARNING_ANONYMOUS_STRUCT_UNION
 DISABLE_WARNING_PADDING_AT_END_OF_STRUCT
 DISABLE_WARNING_ANON_TYPES_IN_ANON_UNION
@@ -195,6 +198,7 @@ struct RayDispatchGlobalData
             dispatchRaysDimensions[1] = umd.GetDispatchHeight();
             dispatchRaysDimensions[2] = umd.GetDispatchDepth();
             uberTilesMap              = umd.GetUberTilesMap();
+
         }
     };
 
@@ -219,18 +223,24 @@ struct RayDispatchGlobalData
             uint64_t callStackHandlerPtr;   // this is the KSP of the continuation handler that is invoked by BTD when the read KSP is 0
             union {
                 uint32_t stackSizePerRay;       // maximal stack size of a ray
-                uint32_t sizePerRay : 8;
-                uint32_t MBZ1       : 24;
+                struct {
+                    uint32_t sizePerRay : 8;
+                    uint32_t MBZ1       : 24;
+                };
             } stack_size_info;
             union {
                 uint32_t numDSSRTStacks;        // number of stacks per DSS
-                uint32_t numRTStacks : 12;
-                uint32_t MBZ2        : 20;
+                struct {
+                    uint32_t numRTStacks : 12;
+                    uint32_t MBZ2        : 20;
+                };
             } num_stacks_info;
             union {
                 uint32_t maxBVHLevels;          // the maximal number of supported instancing levels
-                uint32_t bvhLevels : 3;
-                uint32_t MBZ3      : 29;
+                struct {
+                    uint32_t bvhLevels : 3;
+                    uint32_t MBZ3      : 29;
+                };
             } rt_data_info;
             // In addition to the dword of padding to align `common`, we also
             // add 8 dwords so Xe and Xe3 both have the same RTGlobals size.
@@ -271,16 +281,20 @@ struct RayDispatchGlobalData
             } stack_size_info;
             union {
                 uint32_t numRTStacks;        // number of stacks per DSS
-                uint32_t numDSSRTStacks    : 16;   // number of asynch stacks per DSS
-                uint32_t _pad1_mbz : 16;
+                struct {
+                    uint32_t numDSSRTStacks : 16; // number of asynch stacks per DSS
+                    uint32_t _pad1_mbz : 16;
+                };
 
             } num_stacks_info;
             union {
                 uint32_t packedData;
-                uint32_t maxBVHLevels     : 3;  // the maximal number of supported instancing levels (0->8, 1->1, 2->2, ...)
-                uint32_t hitGroupStride   : 13; // stride of hit group shader records (16-bytes alignment)
-                uint32_t missShaderStride : 13; // stride of miss shader records (8-bytes alignment)
-                uint32_t _pad2_mbz        : 3;
+                struct {
+                    uint32_t maxBVHLevels     : 3;  // the maximal number of supported instancing levels (0->8, 1->1, 2->2, ...)
+                    uint32_t hitGroupStride   : 13; // stride of hit group shader records (16-bytes alignment)
+                    uint32_t missShaderStride : 13; // stride of miss shader records (8-bytes alignment)
+                    uint32_t _pad2_mbz        : 3;
+                };
             } rt_data_info;
             uint32_t flags   : 1;               // per context control flags
             uint32_t pad_mbz : 31;
@@ -307,6 +321,7 @@ static_assert((sizeof(RayDispatchGlobalData::RT::Xe) - sizeof(RayDispatchGlobalD
 static_assert((sizeof(RayDispatchGlobalData::RT::Xe3) - sizeof(RayDispatchGlobalData::RayDispatchGlobalDataCommon)) % 64 == 0, "Unexpected GlobalData alignment");
 
 static_assert(sizeof(RayDispatchGlobalData) == 192, "unexpected size?");
+
 static_assert(sizeof(RayDispatchGlobalData::RT::Xe) == sizeof(RayDispatchGlobalData), "unexpected size?");
 static_assert(sizeof(RayDispatchGlobalData::RT::Xe3) == sizeof(RayDispatchGlobalData), "unexpected size?");
 static_assert(offsetof(RayDispatchGlobalData::RT::Xe, common) == offsetof(RayDispatchGlobalData::RT::Xe3, common), "unexpected size?");
