@@ -144,12 +144,14 @@ CIF_DECLARE_INTERFACE_PIMPL(IgcOclTranslationCtx) : CIF::PimplBase
     OCL_API_CALL static bool SupportsTranslation(CodeType::CodeType_t inType, CodeType::CodeType_t outType){
         static std::pair<CodeType::CodeType_t, CodeType::CodeType_t> supportedTranslations[] =
             {
+// clang-format off
                   // from                 // to
                 { CodeType::elf,      CodeType::llvmBc },
                 { CodeType::elf,      CodeType::oclGenBin },
                 { CodeType::llvmLl,   CodeType::oclGenBin },
                 { CodeType::llvmBc,   CodeType::oclGenBin },
                 { CodeType::spirV,    CodeType::oclGenBin },
+// clang-format on
             };
         for(const auto & st : supportedTranslations){
             if((inType == st.first) && (outType == st.second)){
@@ -181,12 +183,15 @@ CIF_DECLARE_INTERFACE_PIMPL(IgcOclTranslationCtx) : CIF::PimplBase
 
                 TC::DumpShaderFile(pOutputFolder, pInput, inputSize, hash, ".spv", nullptr);
 #if defined(IGC_SPIRV_TOOLS_ENABLED)
-                spv_text spirvAsm = nullptr;
-                if (TC::DisassembleSPIRV(pInput, inputSize, &spirvAsm) == SPV_SUCCESS)
+                if (IGC_IS_FLAG_ENABLED(SpvAsmDumpEnable))
                 {
-                    TC::DumpShaderFile(pOutputFolder, spirvAsm->str, spirvAsm->length, hash, ".spvasm", nullptr);
+                    spv_text spirvAsm = nullptr;
+                    if (TC::DisassembleSPIRV(pInput, inputSize, &spirvAsm) == SPV_SUCCESS)
+                    {
+                        TC::DumpShaderFile(pOutputFolder, spirvAsm->str, spirvAsm->length, hash, ".spvasm", nullptr);
+                    }
+                    spvTextDestroy(spirvAsm);
                 }
-                spvTextDestroy(spirvAsm);
 #endif // defined(IGC_SPIRV_TOOLS_ENABLED)
             }
             llvm::StringRef strInput = llvm::StringRef(pInput, inputSize);
