@@ -6,15 +6,12 @@ SPDX-License-Identifier: MIT
 
 ============================= end_copyright_notice ===========================*/
 
-#include "GenCodeGenModule.h"
 #include "Probe/Assertion.h"
-#include "CLElfLib/ElfReader.h"
 
 #include "DebugInfo/ScalarVISAModule.h"
 #include "DebugInfo/DwarfDebug.hpp"
 #include "DebugInfo/VISADebugInfo.hpp"
 #include "Compiler/CISACodeGen/DebugInfo.hpp"
-#include "Compiler/CISACodeGen/OpenCLKernelCodeGen.hpp"
 #include "llvm/IR/IntrinsicInst.h"
 
 #include "llvmWrapper/IR/IntrinsicInst.h"
