@@ -34,30 +34,30 @@ class IGCVectorizer : public llvm::FunctionPass {
     struct Slice {
         unsigned int OpNum;
         VecArr Vector;
-        Slice* Parent;
+        unsigned ParentIndex;
     };
 
     typedef llvm::SmallVector<Slice, 32> VecOfSlices;
     typedef llvm::SmallVector<VecOfSlices, 3> Tree;
     typedef std::unordered_map<Instruction*, VecArr*> InstructionToSliceMap;
 
-
     struct InsertStruct {
         Instruction* Final = nullptr;
         // contains insert elements
         VecArr Vec;
         // contains slices of vector tree
-        VectorSliceChain Chain;
         VecOfSlices SlChain;
     };
 
     CodeGenContext *CGCtx = nullptr;
+    IGCMD::MetaDataUtils* MDUtils = nullptr;
 
     // when we vectorize, we build a new vector chain,
     // this map contains associations between scalar and vector
     // basically every element inside scalarSlice should point to the same
     // vectorized element which contains all of them
     std::unordered_map<Value*, Value*> ScalarToVector;
+    std::unordered_map<Value*, Value*> ReplacedDictionary;
     InstructionToSliceMap InstructionToSlice;
     // all vector instructions that were produced for chain will be stored
     // in this array, used for clean up if we bail
@@ -68,28 +68,28 @@ class IGCVectorizer : public llvm::FunctionPass {
     std::string LogStr;
     llvm::raw_string_ostream OutputLogStream = raw_string_ostream(LogStr);
     Module* M = nullptr;
+    bool checkIfSIMD16(llvm::Function &F);
     void initializeLogFile(Function& F);
     void writeLog();
 
     void findInsertElementsInDataFlow(llvm::Instruction* I, VecArr& Chain);
-    void collectScalarPath(VecArr& V, VectorSliceChain& Chain);
-    void canonicalizeSlices(VectorSliceChain& Chain);
     bool checkSlice(VecArr& Slice, InsertStruct& InSt);
     bool processChain(InsertStruct& InSt);
-    void clusterInsertElement(InsertElementInst* VecOfInsert, InsertStruct& InSt);
+    void clusterInsertElement(InsertStruct& InSt);
     void collectInstructionToProcess(VecArr& ToProcess, Function& F);
     void buildTree(VecArr &V, VecOfSlices& Chain);
     void printSlice(Slice* S);
 
 
     bool checkPHI(Instruction* Compare, VecArr& Slice);
-    bool handlePHI(VecArr& Slice, Type* VectorType);
+    bool handlePHI(VecArr& Slice);
     bool checkInsertElement(Instruction* First, VecArr& Slice);
     bool handleInsertElement(VecArr& Slice, Instruction* Final);
     bool checkExtractElement(Instruction* Compare, VecArr& Slice);
     bool handleExtractElement(VecArr& Slice);
     bool handleCastInstruction(VecArr& Slice);
     bool handleBinaryInstruction(VecArr& Slice);
+    bool handleIntrinsic(VecArr& Slice);
     bool checkBinaryOperator(VecArr& Slice);
     bool handleIntrinsicInstruction(VecArr& Slice);
 
@@ -98,6 +98,7 @@ class IGCVectorizer : public llvm::FunctionPass {
 
     bool compareOperands(Value* A, Value* B);
     InsertElementInst* createVector(VecArr& Slice, Instruction* InsertPoint);
+    void replaceSliceInstructionsWithExtract(VecArr &Slice, Instruction* CreatedInst);
 
     public:
     llvm::StringRef getPassName() const override { return "IGCVectorizer"; }
@@ -107,6 +108,7 @@ class IGCVectorizer : public llvm::FunctionPass {
     virtual bool runOnFunction(llvm::Function &F) override;
     virtual void getAnalysisUsage(llvm::AnalysisUsage &AU) const override {
         AU.addRequired<CodeGenContextWrapper>();
+        AU.addRequired<MetaDataUtilsWrapper>();
     }
     IGCVectorizer();
     IGCVectorizer(const std::string& FileName);
