@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2020-2024 Intel Corporation
+Copyright (C) 2020-2025 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -11,21 +11,12 @@ SPDX-License-Identifier: MIT
 #include "Compiler/IGCPassSupport.h"
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/IR/Function.h>
-#include <llvmWrapper/IR/Instructions.h>
 #include "common/LLVMWarningsPop.hpp"
-#ifndef DX_ONLY_IGC
-#ifndef VK_ONLY_IGC
-#include <AdaptorOCL/SPIRV/SPIRVInternal.h>
-#endif //#ifndef VK_ONLY_IGC
-#endif //#ifndef DX_ONLY_IGC
 #include "MDFrameWork.h"
 #include "Probe/Assertion.h"
 
-#ifndef DX_ONLY_IGC
-#ifndef VK_ONLY_IGC
-using namespace igc_spv;
-#endif //#ifndef VK_ONLY_IGC
-#endif //#ifndef DX_ONLY_IGC
+static const unsigned SPIRAS_Local = 3;
+
 using namespace llvm;
 using namespace IGC;
 using namespace IGC::IGCMD;
@@ -47,7 +38,7 @@ const char* NamedBarriersResolution::NAMED_BARRIERS_BARRIER_ARG3 = "_Z24work_gro
 
 const int NamedBarriersResolution::GetMaxNamedBarriers()
 {
-    if (NamedBarrierHWSupport())
+    if (NamedBarrierHWSupport(m_GFX_CORE))
     {
         // User can define only 32 named barriers,
         // The TG barrier is an alias for named barrier 0
@@ -126,7 +117,7 @@ bool NamedBarriersResolution::runOnModule(Module& M)
         }
     }
 
-    if (NamedBarrierHWSupport())
+    if (NamedBarrierHWSupport(m_GFX_CORE))
     {
         // Remove not needed wrapper for Init NBarrier built-in
         for (const auto& [barrierStruct, barrierData] : m_MapInitToID)
@@ -214,7 +205,7 @@ void NamedBarriersResolution::HandleNamedBarrierInitHW(CallInst& NBarrierInitCal
     IGC_ASSERT_MESSAGE(m_CountNamedBarriers <= GetMaxNamedBarriers(), "NamedBarriersResolution : We crossed the max of amount of named barriers!");
     Module* module = NBarrierInitCall.getModule();
 
-    Value* threadGroupNBarrierID = (Value*)getInt32(module, m_CountNamedBarriers);
+    Value* threadGroupNBarrierID = (Value*)ConstantInt::get(Type::getInt32Ty(module->getContext()), m_CountNamedBarriers, true);
     Value* pointerToNBarrierStruct = FindAllocStructNBarrier((Value*)(&NBarrierInitCall), true);
 
     s_namedBarrierInfo structNb;
@@ -243,8 +234,6 @@ void NamedBarriersResolution::HandleNamedBarrierSyncHW(CallInst& NBarrierSyncCal
     Value* falseValue = IRB.getInt1(false);
     Value* gpuScopeValue = IRB.getInt32(LSC_SCOPE_GPU);
     Value* groupScopeValue = IRB.getInt32(LSC_SCOPE_GROUP);
-    // Producer and Consumer
-    Value* barrierType = IRB.getInt16(0);
 
     ConstantInt* memFenceType = cast<ConstantInt>(NBarrierSyncCall.getArgOperand(1));
     // LOCAL = 1
@@ -270,17 +259,8 @@ void NamedBarriersResolution::HandleNamedBarrierSyncHW(CallInst& NBarrierSyncCal
         "",
         &(NBarrierSyncCall));
 
-    GenIntrinsicInst::Create(
-        GenISAIntrinsic::getDeclaration(module, GenISAIntrinsic::GenISA_threadgroupnamedbarriers_signal),
-        { nbStruct.threadGroupNBarrierID, barrierType, nbStruct.threadGroupNBarrierCount,  nbStruct.threadGroupNBarrierCount },
-        "",
-        &(NBarrierSyncCall));
-
-    GenIntrinsicInst::Create(
-        GenISAIntrinsic::getDeclaration(module, GenISAIntrinsic::GenISA_threadgroupnamedbarriers_wait),
-        { nbStruct.threadGroupNBarrierID },
-        "",
-        &(NBarrierSyncCall));
+    CallSignal(nbStruct.threadGroupNBarrierID, nbStruct.threadGroupNBarrierCount, nbStruct.threadGroupNBarrierCount, NamedBarrierType::ProducerConsumer, &NBarrierSyncCall);
+    CallWait(nbStruct.threadGroupNBarrierID, &NBarrierSyncCall);
 
     NBarrierSyncCall.eraseFromParent();
 }
@@ -341,8 +321,15 @@ void NamedBarriersResolution::HandleNamedBarrierInitSW(CallInst& NBarrierInitCal
         Type::getInt32PtrTy(context, SPIRAS_Local)
     };
     Type* BaseTy = m_NamedBarrierArray->getValueType();
-    auto pointerNBarrier = GetElementPtrInst::Create(BaseTy, m_NamedBarrierArray, { getInt64(module, 0), getInt32(module, 0) }, "", &(NBarrierInitCall));
-    auto bitcastPointerNBarrier = BitCastInst::CreatePointerBitCastOrAddrSpaceCast(pointerNBarrier, m_NamedBarrierType->getPointerTo(SPIRAS_Local), "", &(NBarrierInitCall));
+    auto pointerNBarrier = GetElementPtrInst::Create(
+        BaseTy, m_NamedBarrierArray,
+        {ConstantInt::get(Type::getInt64Ty(module->getContext()), 0, true),
+         ConstantInt::get(Type::getInt32Ty(module->getContext()), 0, true)},
+        "", &(NBarrierInitCall));
+    auto bitcastPointerNBarrier =
+        BitCastInst::CreatePointerBitCastOrAddrSpaceCast(
+            pointerNBarrier, m_NamedBarrierType->getPointerTo(SPIRAS_Local), "",
+            &(NBarrierInitCall));
     SmallVector<Value*, 3> ArgsVal
     {
         NBarrierInitCall.getArgOperand(0),
@@ -411,23 +398,55 @@ void NamedBarriersResolution::visitCallInst(CallInst& CI)
 
     if (isNamedBarrierInit(funcName))
     {
-        NamedBarrierHWSupport()
+        NamedBarrierHWSupport(m_GFX_CORE)
             ? HandleNamedBarrierInitHW(CI)
             : HandleNamedBarrierInitSW(CI);
     }
     else if (isNamedBarrierSync(funcName))
     {
-        NamedBarrierHWSupport()
+        NamedBarrierHWSupport(m_GFX_CORE)
             ? HandleNamedBarrierSyncHW(CI)
             : HandleNamedBarrierSyncSW(CI);
     }
 }
 
 
-bool NamedBarriersResolution::NamedBarrierHWSupport()
+bool NamedBarriersResolution::NamedBarrierHWSupport(GFXCORE_FAMILY GFX_CORE)
 {
-    bool hwSupport = m_GFX_CORE == IGFX_XE_HPC_CORE;
-    hwSupport |= m_GFX_CORE == IGFX_XE2_HPG_CORE;
-    hwSupport |= m_GFX_CORE == IGFX_XE3_CORE;
+    bool hwSupport = GFX_CORE == IGFX_XE_HPC_CORE;
+    hwSupport |= GFX_CORE == IGFX_XE2_HPG_CORE;
+    hwSupport |= GFX_CORE == IGFX_XE3_CORE;
     return hwSupport;
+}
+
+void NamedBarriersResolution::CallSignal(llvm::Value* barrierID, llvm::Value* ProducerCnt, llvm::Value* ConsumerCnt, NamedBarrierType Type, llvm::Instruction* pInsertBefore)
+{
+    IGCLLVM::IRBuilder<> builder(pInsertBefore);
+    llvm::Module* pM = pInsertBefore->getModule();
+
+    llvm::Value* namedBarrierType = builder.getInt16((int)Type);
+
+    llvm::Value* getIDInt8 = llvm::BitCastInst::CreateIntegerCast(barrierID, builder.getInt8Ty(), false, "", pInsertBefore);
+    llvm::Value* getProducerCntInt8 = llvm::BitCastInst::CreateIntegerCast(ProducerCnt, builder.getInt8Ty(), false, "", pInsertBefore);
+    llvm::Value* getConsumerCntInt8 = llvm::BitCastInst::CreateIntegerCast(ConsumerCnt, builder.getInt8Ty(), false, "", pInsertBefore);
+
+    GenIntrinsicInst::Create(
+        GenISAIntrinsic::getDeclaration(pM, GenISAIntrinsic::GenISA_threadgroupnamedbarriers_signal),
+        { getIDInt8, namedBarrierType, getProducerCntInt8, getConsumerCntInt8 },
+        "",
+        pInsertBefore);
+}
+
+void NamedBarriersResolution::CallWait(llvm::Value* barrierID, llvm::Instruction* pInsertBefore)
+{
+    IGCLLVM::IRBuilder<> builder(pInsertBefore);
+    llvm::Module* pM = pInsertBefore->getModule();
+
+    llvm::Value* getIDInt8 = llvm::BitCastInst::CreateIntegerCast(barrierID, builder.getInt8Ty(), false, "", pInsertBefore);
+
+    GenIntrinsicInst::Create(
+        GenISAIntrinsic::getDeclaration(pM, GenISAIntrinsic::GenISA_threadgroupnamedbarriers_wait),
+        { getIDInt8 },
+        "",
+        pInsertBefore);
 }
\ No newline at end of file
