@@ -23,7 +23,6 @@ SPDX-License-Identifier: MIT
 #include "llvm/Transforms/Utils/Local.h"
 #include "llvm/IR/DataLayout.h"
 #include "llvm/IR/DebugInfo.h"
-#include "llvm/IR/DebugInfoMetadata.h"
 #include "llvm/IR/Dominators.h"
 #include <llvmWrapper/ADT/Optional.h>
 #include "common/LLVMWarningsPop.hpp"
@@ -375,7 +374,12 @@ bool PrivateMemoryResolution::runOnModule(llvm::Module& M)
                 for (auto AI = childF->arg_begin(), AE = childF->arg_end(); AI != AE; ++AI)
                 {
                     // Argument offsets are also OWORD aligned
-                    argSize += iSTD::Align(static_cast<DWORD>(DL.getTypeAllocSize(AI->getType())), SIZE_OWORD);
+                    auto aiTy = AI->getType();
+
+                    if (aiTy->isMetadataTy())
+                        continue;
+
+                    argSize += iSTD::Align(static_cast<DWORD>(DL.getTypeAllocSize(aiTy)), SIZE_OWORD);
                 }
                 // Also do it for return value
                 if (!childF->getReturnType()->isVoidTy())
@@ -520,6 +524,11 @@ static void sinkAllocas(SmallVectorImpl<AllocaInst*>& Allocas) {
         for (unsigned i = 1; i < UInsts.size(); ++i) {
             Instruction* Use = UInsts[i];
             BasicBlock* UseBB = Use->getParent();
+
+            // skip unreachable BB
+            if (UseBB->use_empty() && !UseBB->isEntryBlock())
+                continue;
+
             DomBB = DT.findNearestCommonDominator(DomBB, UseBB);
             if (!DomBB) {
                 break;
@@ -594,6 +603,11 @@ static void sinkAllocaSingleUse(SmallVectorImpl<AllocaInst*>& Allocas) {
             for (unsigned i = 1; i < UInsts.size(); ++i) {
                 Instruction* Use = UInsts[i];
                 BasicBlock* UseBB = Use->getParent();
+
+                // skip unreachable BB
+                if (UseBB->use_empty() && !UseBB->isEntryBlock())
+                    continue;
+
                 DomBB = DT.findNearestCommonDominator(DomBB, UseBB);
                 if (!DomBB) {
                     break;
@@ -1259,9 +1273,25 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
     // Only OCL is supposed to reach here.
     IGC_ASSERT_EXIT(ShaderType::OPENCL_SHADER == Ctx.type);
 
+    // Save the insert point to ensure appropriate sequence of instructions after getImplicitArgValue(),
+    // because getImplicitArgValue() can move instructions, and it means that the insert point will be moved too.
+    Instruction* pointInstr = &*entryBuilder.GetInsertPoint();
+    if (pointInstr->isDebugOrPseudoInst())
+        pointInstr = pointInstr->getNextNonDebugInstruction();
+    if (GenIntrinsicInst* inst = dyn_cast_or_null<GenIntrinsicInst>(pointInstr))
+    {
+        if (inst->getIntrinsicID() == GenISAIntrinsic::GenISA_getR0 ||
+            inst->getIntrinsicID() == GenISAIntrinsic::GenISA_getPrivateBase)
+            pointInstr = inst->getNextNonDebugInstruction();
+    }
+
     // Find the implicit argument representing r0 and the private memory base.
     Value* r0Val = implicitArgs.getImplicitArgValue(*m_currFunction, ImplicitArg::R0, m_pMdUtils);
     Value* privateMemPtr = implicitArgs.getImplicitArgValue(*m_currFunction, ImplicitArg::PRIVATE_BASE, m_pMdUtils);
+
+    // Restore insert point saved before getImplicitArgValue()
+    entryBuilder.SetInsertPoint(pointInstr);
+
     // Note: for debugging purposes privateMemPtr will be marked as Output to keep its liveness all time
 
     // Resolve the call
