@@ -259,6 +259,24 @@ public:
         llvm::Value* srcAddressW,
         llvm::Value* lod);
 
+    llvm::Value* Create_typedwriteMS(
+        llvm::Value* dstBuffer,
+        llvm::Value* srcAddressU,
+        llvm::Value* srcAddressV,
+        llvm::Value* srcAddressW,
+        llvm::Value* sampleIdx,
+        llvm::Value* float_X,
+        llvm::Value* float_Y,
+        llvm::Value* float_Z,
+        llvm::Value* float_W);
+
+    llvm::Value* Create_typedreadMS(
+        llvm::Value* dstBuffer,
+        llvm::Value* srcAddressU,
+        llvm::Value* srcAddressV,
+        llvm::Value* srcAddressW,
+        llvm::Value* sampleIdx);
+
     llvm::Value* Create_typedread_msaa2D(
         llvm::Value* srcBuffer,
         llvm::Value* sampleIdx,
