@@ -122,6 +122,7 @@ struct CompileOptions {
   bool DumpIR = false;
   bool DumpAsm = false;
   bool DumpDebugInfo = false;
+  bool DumpInstOffset = false;
 
   bool TimePasses = false;
   bool ShowStats = false;
@@ -168,6 +169,8 @@ struct CompileOptions {
 
   unsigned DepressurizerGRFThreshold = 2560;
   unsigned DepressurizerFlagGRFTolerance = 3840;
+
+  bool ReportLSCStoresWithNonDefaultL1CacheControls = false;
 };
 
 struct ExternalData {
