@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2023 Intel Corporation
+Copyright (C) 2017-2025 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -13,14 +13,12 @@ SPDX-License-Identifier: MIT
 #include <llvm/IR/Constants.h>
 #include <llvm/ADT/StringRef.h>
 #include <llvm/Support/Casting.h>
-#include <llvm/ADT/StringSwitch.h>
 #include <llvm/ADT/MapVector.h>
 #include "common/LLVMWarningsPop.hpp"
 
 #include "StringMacros.hpp"
 #include "common/igc_regkeys.hpp"
 
-#include <iostream>
 #include <optional>
 
 using namespace llvm;
@@ -186,8 +184,10 @@ MDNode* CreateNode(const std::vector<val> &vec, Module* module, StringRef name)
     for (auto it = vec.begin(); it != vec.end(); ++it)
     {
         nodes.push_back(CreateNode(*(it), module, name.str() + "Vec[" + std::to_string(i++) + "]"));
-        if (IGC_IS_FLAG_ENABLED(ShaderDumpEnable) && IGC_IS_FLAG_DISABLED(ShowFullVectorsInShaderDumps) && i > MAX_VECTOR_SIZE_TO_PRINT_IN_SHADER_DUMPS)
+        if (IGC_IS_FLAG_DISABLED(ShowFullVectorsInShaderDumps) && i > MAX_VECTOR_SIZE_TO_PRINT_IN_SHADER_DUMPS)
         {
+          if (IGC_IS_FLAG_ENABLED(ShaderDumpEnable))
+          {
             std::string flagName = "ShowFullVectorsInShaderDumps";
             #ifndef _WIN32
                 flagName = "IGC_" + flagName;
@@ -205,7 +205,8 @@ MDNode* CreateNode(const std::vector<val> &vec, Module* module, StringRef name)
                 printWarningFirstTime = false;
             }
             nodes.push_back(CreateNode(false, module, warningMessage + " " + flagName + " currently equals"));
-            break;
+          }
+          break;
         }
     }
     MDNode* node = MDNode::get(module->getContext(), nodes);
@@ -561,6 +562,7 @@ bool IGC::isCallStackHandler(const IGC::FunctionMetaData &funcMD)
     return funcMD.rtInfo.callableShaderType == IGC::CallStackHandler;
 }
 
+
 int IGC::extractAnnotatedNumThreads(const IGC::FunctionMetaData& funcMD)
 {
     static constexpr const char* searchedString = "num-thread-per-eu";
