@@ -19,7 +19,7 @@ static bool regSortCompare(regMap map1, regMap map2) {
   return false;
 }
 
-static bool regSortCompareBeforeRA(regMapBRA map1, regMapBRA map2) {
+static bool regSortCompareAfterRA(regMapBRA map1, regMapBRA map2) {
   if (map1.opndNum < map2.opndNum) {
     return true;
   } else if (map1.opndNum > map2.opndNum) {
@@ -430,7 +430,7 @@ void SRSubPass::SRSub(G4_BB *bb) {
 
 // Check if current instruction is the candidate of sendi.
 // Recorded as candidate.
-bool SRSubPassBeforeRA::isSRCandidateBeforeRA(G4_INST *inst,
+bool SRSubPassAfterRA::isSRCandidateAfterRA(G4_INST *inst,
                                               regCandidatesBRA &dstSrcRegs) {
   if (!inst->isSend()) {
     return false;
@@ -482,6 +482,7 @@ bool SRSubPassBeforeRA::isSRCandidateBeforeRA(G4_INST *inst,
   int movInstNum = 0;
   int32_t firstDefID = 0x7FFFFFFF; // the ID of the first instruction define the
   std::vector<std::pair<Gen4_Operand_Number, unsigned>> notRemoveableMap;
+  std::vector<G4_INST *> immMovs;
   for (auto I = inst->def_begin(), E = inst->def_end(); I != E; ++I) {
     auto &&def = *I;
 
@@ -500,6 +501,11 @@ bool SRSubPassBeforeRA::isSRCandidateBeforeRA(G4_INST *inst,
         return false;
       }
 
+      // The instruction is only used for payload preparation.
+      if (i->use_size() != 1) {
+        return false;
+      }
+
       G4_DstRegRegion *dstRgn = i->getDst();
       G4_Operand *src = i->getSrc(0);
 
@@ -565,14 +571,15 @@ bool SRSubPassBeforeRA::isSRCandidateBeforeRA(G4_INST *inst,
         return false;
       }
 
-      // The instruction is only used for payload preparation.
-      if (i->use_size() != 1) {
+      // Move to self instruction will be removed by following pass. So cannot
+      // be counted.
+      if (dstRgn->getLinearizedStart() == src->getLinearizedStart()) {
         return false;
       }
 
       // It's not global define
       if (!(builder.getIsKernel() && kernel.fg.getNumBB() == 1)) {
-        if (kernel.fg.globalOpndHT.isOpndGlobal(dstRgn)) {
+        if (kernel.fg.globalOpndHT.isOpndGlobal(dstRgn) && !dstRgn->getTopDcl()->getIsBBLocal()) {
           return false;
         }
       }
@@ -580,6 +587,86 @@ bool SRSubPassBeforeRA::isSRCandidateBeforeRA(G4_INST *inst,
       return true;
     };
 
+    // mov (16)             r81.0<1>:f  0x8:f // $52:&54:
+    // mov (16|M16)         r89.0<1>:f  0x8:f // $53:&55:
+    // mov (16)             r82.0<1>:f  0x0:f // $54:&56:
+    // mov (16|M16)         r90.0<1>:f  0x0:f // $55:&57:
+    // mov (16)             r83.0<1>:f  0x0:f // $56:&58:
+    // mov (16|M16)         r91.0<1>:f  0x0:f // $57:&59:
+    // mov (16)             r84.0<1>:f  0x0:f // $58:&60:
+    // mov (16|M16)         r92.0<1>:f  0x0:f // $59:&61:
+    // mov (16)             r85.0<1>:f  0x0:f // $60:&62:
+    // mov (16|M16)         r93.0<1>:f  0x0:f // $61:&63:
+    // mov (16)             r86.0<1>:f  0x0:f // $62:&64:
+    // mov (16|M16)         r94.0<1>:f  0x0:f // $63:&65:
+    // mov (16)             r87.0<1>:f  0x0:f // $64:&66:
+    // mov (16|M16)         r95.0<1>:f  0x0:f // $65:&67:
+    // mov (16)             r88.0<1>:f  0x0:f // $66:&68:
+    // mov (16|M16)         r96.0<1>:f  0x0:f // $67:&69:
+    // ==>
+    // mov (16)             r81.0<1>:f  0x8:f // $52:&54:
+    // mov (16|M16)         r89.0<1>:f  0x8:f // $53:&55:
+    // mov (16)             r82.0<1>:f  0x0:f // $54:&56:
+    // mov (16|M16)         r90.0<1>:f  0x0:f // $55:&57:
+    //
+    // Reuse r81, r89, r82, r90 in the gather send
+    auto getRemoveableImm = [this](G4_INST *inst,
+                                   std::vector<G4_INST *> &immMovs) {
+      // The instruction is only used for payload preparation.
+      if (inst->use_size() != 1) {
+        return (G4_INST *)nullptr;
+      }
+
+      if (inst->opcode() != G4_mov) {
+        return (G4_INST *)nullptr;
+      }
+
+      G4_DstRegRegion *dst = inst->getDst();
+      // dst GRF aligned and contigous
+      if (dst->getSubRegOff() || dst->getHorzStride() != 1) {
+        return (G4_INST *)nullptr;
+      }
+
+      if (kernel.fg.globalOpndHT.isOpndGlobal(dst)) {
+        return (G4_INST *)nullptr;
+      }
+
+      // GRF Alignment with physical register assigned
+      if (dst->getLinearizedStart() % builder.getGRFSize() != 0) {
+        return (G4_INST *)nullptr;
+      }
+
+      // If the destination operand size is less than 1 GRF
+      if ((dst->getLinearizedEnd() - dst->getLinearizedStart() + 1) <
+          builder.getGRFSize()) {
+        return (G4_INST *)nullptr;
+      }
+
+      G4_Operand *src = inst->getSrc(0);
+      int64_t imm = src->asImm()->getImm();
+      for (size_t i = 0; i < immMovs.size(); i++) {
+        G4_INST *imov = immMovs[i];
+        G4_Operand *isrc = imov->getSrc(0);
+        int64_t iimm = isrc->asImm()->getImm();
+        if (imm == iimm &&
+            src->getType() == isrc->getType() && // Same value and same type
+            inst->getDst()->getType() ==
+                imov->getDst()->getType() && // Same dst type
+            inst->getDst()->asDstRegRegion()->getHorzStride() ==
+                imov->getDst()
+                    ->asDstRegRegion()
+                    ->getHorzStride() &&                  // Same region
+            inst->getExecSize() == imov->getExecSize() && // Same execution size
+            inst->getMaskOffset() ==
+                imov->getMaskOffset()) { // Same mask offset
+          return imov;
+        }
+      }
+      immMovs.push_back(inst);
+
+      return (G4_INST *)nullptr;
+    };
+
     //if opndNum + offset is defined multiple times, cannobe be removed
     G4_Operand *dst = movInst->getDst();
     unsigned offset = dst->getLeftBound() / builder.getGRFSize();
@@ -604,6 +691,23 @@ bool SRSubPassBeforeRA::isSRCandidateBeforeRA(G4_INST *inst,
         movInstNum++;
       }
     } else {
+      if (movInst->opcode() == G4_mov && movInst->getSrc(0) &&
+          movInst->getSrc(0)->isImm()) {
+        // Check if there is mov instruction with same imm value
+        G4_INST *lvnMov = getRemoveableImm(movInst, immMovs);
+
+        if (lvnMov) {
+          // The offset is the offset of original dst, which is used to identify
+          // the original register used in send.
+          // The opndNum is the opndNum of send.
+          regMapBRA regPair(movInst, opndNum, offset,
+                            lvnMov->getDst()); // the lvn mov dst can be reused
+          dstSrcRegs.dstSrcMap.push_back(regPair);
+          firstDefID = std::min(firstDefID, def.first->getLocalId());
+          movInstNum++;
+          continue;
+        }
+      }
       notRemoveableMap.push_back(std::make_pair(opndNum, offset));
     }
   }
@@ -639,14 +743,14 @@ bool SRSubPassBeforeRA::isSRCandidateBeforeRA(G4_INST *inst,
   dstSrcRegs.firstDefID = firstDefID;
   // Sort according to the register order in the original payload
   std::sort(dstSrcRegs.dstSrcMap.begin(), dstSrcRegs.dstSrcMap.end(),
-            regSortCompareBeforeRA);
+            regSortCompareAfterRA);
 
   return true;
 }
 
 // Replace the send instruction with the payload of
 // Insert the scalar register intialization mov instructions.
-bool SRSubPassBeforeRA::replaceWithSendiBeforeRA(G4_BB *bb,
+bool SRSubPassAfterRA::replaceWithSendiAfterRA(G4_BB *bb,
                                                  INST_LIST_ITER instIter,
                                                  regCandidatesBRA &dstSrcRegs) {
   G4_INST *inst = *instIter;
@@ -784,7 +888,7 @@ bool SRSubPassBeforeRA::replaceWithSendiBeforeRA(G4_BB *bb,
   return true;
 }
 
-void SRSubPassBeforeRA::SRSubBeforeRA(G4_BB *bb) {
+void SRSubPassAfterRA::SRSubAfterRA(G4_BB *bb) {
   bb->resetLocalIds();
 
   class CmpFirstDef {
@@ -803,7 +907,7 @@ void SRSubPassBeforeRA::SRSubBeforeRA(G4_BB *bb) {
     G4_INST *inst = *ii;
 
     regCandidatesBRA dstSrcRegs;
-    if (!isSRCandidateBeforeRA(inst, dstSrcRegs)) {
+    if (!isSRCandidateAfterRA(inst, dstSrcRegs)) {
       ii++;
       dstSrcRegs.dstSrcMap.clear();
       continue;
@@ -840,12 +944,26 @@ void SRSubPassBeforeRA::SRSubBeforeRA(G4_BB *bb) {
     candidatesIt = candidates.find(inst);
     //Is candidate send
     if (candidatesIt != candidates.end()) {
-      bool overwrite = false;
       // Scan backward from the send instruction.
       INST_LIST_RITER scan_ri = ri;
       scan_ri++;
       G4_INST *rInst = *scan_ri;
+
       while (rInst->getLocalId() > candidates[inst].firstDefID) {
+        if (rInst->isDead()) {
+          // If the inst is marked as dead, it's dst will not kill other value
+          //  Such as in following case, if third instruction is removed, r64
+          //  value of first instruction is kept.
+          // mov (16)             r16.0<1>:ud  r64.0<1;1,0>:ud // $214:&226:
+          // mov (16)             r17.0<1>:ud  r66.0<1;1,0>:ud // $216:&228:
+          // mov (16)             r64.0<1>:ud  r68.0<1;1,0>:ud // $218:&230:
+          scan_ri++;
+          if (scan_ri == rend) {
+            break;
+          }
+          rInst = *scan_ri;
+          continue;
+        }
         G4_Operand *dst = rInst->getDst();
         if (dst && !dst->isNullReg()) {
           G4_VarBase *base = dst->getBase();
@@ -853,7 +971,7 @@ void SRSubPassBeforeRA::SRSubBeforeRA(G4_BB *bb) {
               (base->isRegVar()) ? base->asRegVar()->getPhyReg() : base;
 
           if (phyReg->getKind() == G4_VarBase::VK_phyGReg) {
-            // If the register reuse happened in the mov instruction which will
+            // If the register redefined happened in the mov instruction which will
             // be removed. Or the instruction whose dst will be accessed through
             // s0. It's not real reuse. Such as the r8 in following
             // instructions.
@@ -862,34 +980,36 @@ void SRSubPassBeforeRA::SRSubBeforeRA(G4_BB *bb) {
             // mov(8)  r10.0<1>:f  r3.0<8; 8, 1>:f
             // mov(8)  r11.0<1>:f  r3.0<8; 8, 1>:f
             // mov(8)  r7.0 <1>:d  r5.0<1; 1, 0>:d
-            // mov(8)  r8.0 <1>:f  r3.0<1; 1, 0>:f
+            // mov(8)  r8.0 <1>:f  r10.0<1; 1, 0>:f //assume cannot be removed
             // sends(8) null:ud r112 r7 0x14c : ud 0x2035093 : ud // typed
             // surface write, resLen=0, msgLen=1, extMsgLen=5
             // clang-format on
-            bool sameInst = false;
-            for (int i = 0; i < (int)candidates[inst].dstSrcMap.size(); i++) {
-              if (rInst->getLocalId() ==
-                  candidates[inst].dstSrcMap[i].inst->getLocalId()) {
-                sameInst = true;
-                break;
+            G4_Operand *dst = rInst->getDst();
+            unsigned short dstRegLB = dst->getLinearizedStart();
+            unsigned short dstRegRB = dst->getLinearizedEnd();
+
+            // There is any none removeable offset, the offset define move
+            // cannot be removed.
+            std::vector<regMapBRA>::iterator dstSrcRegsIter;
+            for (dstSrcRegsIter = candidates[inst].dstSrcMap.begin();
+                 dstSrcRegsIter != candidates[inst].dstSrcMap.end();) {
+              std::vector<regMapBRA>::iterator nextIter = dstSrcRegsIter;
+              nextIter++;
+
+              // If reverse scan instruction local ID is equal or less than removed
+              // instruction local ID, there is no need to check.
+              if (rInst->getLocalId() <= (*dstSrcRegsIter).inst->getLocalId()) {
+                dstSrcRegsIter = nextIter;
+                continue;
               }
-            }
-
-            if (!sameInst) {
-              G4_Operand *dst = rInst->getDst();
-              unsigned short dstRegLB = dst->getLinearizedStart();
-              unsigned short dstRegRB = dst->getLinearizedEnd();
-              for (int i = 0; i < (int)candidates[inst].dstSrcMap.size(); i++) {
-                int srcRegLB =
-                    candidates[inst].dstSrcMap[i].opnd->getLinearizedStart();
-                int srcRegRB =
-                    candidates[inst].dstSrcMap[i].opnd->getLinearizedEnd();
-
-                if (!(srcRegRB < dstRegLB || srcRegLB > dstRegRB)) {
-                  // Register is reused.
-                  overwrite = true;
-                  break;
-                }
+              int srcRegLB = (*dstSrcRegsIter).opnd->getLinearizedStart();
+              int srcRegRB = (*dstSrcRegsIter).opnd->getLinearizedEnd();
+              if (!(srcRegRB < dstRegLB || srcRegLB > dstRegRB)) {
+                // Register is redefined.
+                dstSrcRegsIter =
+                    candidates[inst].dstSrcMap.erase(dstSrcRegsIter);
+              } else {
+                dstSrcRegsIter = nextIter;
               }
             }
           }
@@ -900,22 +1020,24 @@ void SRSubPassBeforeRA::SRSubBeforeRA(G4_BB *bb) {
         }
         rInst = *scan_ri;
       }
-      if (overwrite) {
+
+      // Due to extra mov for s0, so don't use s0 if equal or less than 1 mov
+      // inst can be removed.
+      if (candidates[inst].dstSrcMap.size() <= 1 &&
+          builder.getuint32Option(vISA_EnableGatherWithImmPreRA) !=
+              INDIRECT_TYPE::ALWAYS_S0) {
         candidates.erase(candidatesIt);
+      } else {
+        for (int j = 0; j < (int)candidatesIt->second.dstSrcMap.size(); j++) {
+          G4_INST *movInst = candidatesIt->second.dstSrcMap[j].inst;
+          movInst->markDead();
+        }
       }
     }
 
     ri++;
   }
 
-  for (candidatesIt = candidates.begin(); candidatesIt != candidates.end();
-       candidatesIt++) {
-    for (int i = 0; i < (int)candidatesIt->second.dstSrcMap.size(); i++) {
-      G4_INST *movInst = candidatesIt->second.dstSrcMap[i].inst;
-      movInst->markDead();
-    }
-  }
-
   // Replace the send instruction with sendi
   // Remove the mov instructions that marked as dead
   INST_LIST_ITER iter;
@@ -926,7 +1048,7 @@ void SRSubPassBeforeRA::SRSubBeforeRA(G4_BB *bb) {
 
     candidatesIt = candidates.find(inst);
     if (candidatesIt != candidates.end()) {
-      replaceWithSendiBeforeRA(bb, curIter, candidates[inst]);
+      replaceWithSendiAfterRA(bb, curIter, candidates[inst]);
     }
     if (inst->isDead()) {
       bb->erase(curIter);
